Unit Setup;
{========================================================================}
Interface
  Uses
    MfmDefs;
  Procedure ParseCommandLine;
  Procedure BuildSkipList;
  Procedure ParseConfigFile;
  Function OkToAdd(InString : String) : Boolean;
  Function CommentEntry : Boolean;
  Procedure FindOrphans;
  Procedure BuildList;
  Function Bytes(NumberOfBytes : LongInt) : S8;
  Procedure SetupScreen;
  Procedure ReDrawScreen;
{========================================================================}
Implementation
  Uses
    Area, Crt, Display, Dos, MaxAreas, Screen, Strings;
{========================================================================}
Procedure ParseCommandLine;
  Var
    Pclb : Byte;
    FileAreaPathOk, AreaPathOk, OutputSelected : Boolean;
  Begin
    ReDirectTo := StandardIO; FileAreaPath := ''; Columns := 5; ColumnPos := 16;
    FileAreaPathOk := False; AreaPathOk := False; OutputSelected := False;
    MfmRunFb := False; AreaChanged := False;
    Assign(Input,''); Reset(Input);
    Assign(Output,''); Rewrite(Output);
    WriteLn(Pgmid); WriteLn;
    If ParamCount = 0 Then
    Begin
      ReDirectTo := Console;
      OutputSelected := True;
      AssignCrt(Input); Reset(Input);
      AssignCrt(Output); Rewrite(Output);
      Fsplit(Fexpand(ParamStr(0)),D,N,E);
      AreaPath := D+'AREA.DAT';
      GetAreaTable;
      If NumberOfAreaEntries < Columns Then Columns := NumberOfAreaEntries;
      SetupScreen;
      FileAreaPath := ChooseArea;
      If (FileAreaPath = 'QUIT') Or (FileAreaPath = 'QUITQUICK') Then
      Begin
        Halt(250);
      End;
    End
    Else
    Begin
      For Pclb := 1 To ParamCount Do
      Begin
        If Copy(UpperString(ParamStr(Pclb)),1,2) = '-A' Then
        Begin
          AreaPathOk := True;
          AreaPath := Copy(UpperString(ParamStr(Pclb)),3,Length(ParamStr(Pclb))-2);
          If Length(AreaPath) > 0 Then
          Begin
            FindFirst(AreaPath,Archive,DirInfo);
            If DosError <> 0 Then
            Begin
              If Copy(AreaPath,Length(AreaPath),1) <> '\' Then AreaPath := AreaPath + '\';
              FindFirst(AreaPath+'AREA.DAT',Archive,DirInfo);
              If DosError <> 0 Then
              Begin
                WriteLn('AREA.DAT not found in '+AreaPath+' !');
                Halt(1);
              End
              Else
              Begin
                AreaPath := AreaPath + 'AREA.DAT';
              End;
            End;
          End;
        End;
        If Copy(UpperString(ParamStr(Pclb)),1,2) = '-C' Then
        Begin
          OutputSelected := True;
          If Copy(ParamStr(Pclb),3,1) = '0' Then
          Begin
            ReDirectTo := Console;
            AssignCrt(Input); Reset(Input);
            AssignCrt(Output); Rewrite(Output);
          End;
          If Copy(ParamStr(Pclb),3,1) = '1' Then
          Begin
            ReDirectTo := ComPort1;
            Assign(Input,'Com1'); Reset(Input);
            Assign(Output,'Com1'); Rewrite(Output);
          End;
          If Copy(ParamStr(Pclb),3,1) = '2' Then
          Begin
            ReDirectTo := ComPort2;
            Assign(Input,'Com2'); Reset(Input);
            Assign(Output,'Com2'); Rewrite(Output);
          End;
          If Copy(ParamStr(Pclb),3,1) = '9' Then
          Begin
            ReDirectTo := StandardIO;
            Assign(Input,''); Reset(Input);
            Assign(Output,''); Rewrite(Output);
          End;
        End;
        If Copy(UpperString(ParamStr(Pclb)),1,2) = '-P' Then
        Begin
          FileAreaPathOk := True;
          FileAreaPath := Copy(UpperString(ParamStr(Pclb)),3,Length(ParamStr(Pclb))-2);
          If Length(FileAreaPath) > 0 Then
          Begin
            If Copy(FileAreaPath,Length(FileAreaPath),1) <> '\' Then FileAreaPath := FileAreaPath + '\';
            FindFirst(FileAreaPath+'*.*',AnyFile,DirInfo);
            If DosError <> 0 Then
            Begin
              WriteLn('Directory '+FileAreaPath+' not found!');
              Halt(1);
            End;
          End;
        End;
        If Copy(UpperString(ParamStr(Pclb)),1,2) = '-K' Then
        Begin
          Val(Copy(ParamStr(Pclb),3,1),Columns,Result);
          If Columns < 3 Then Columns := 3;
          If Columns > 8 Then Columns := 8;
        End;
        If Copy(UpperString(ParamStr(Pclb)),1,2) = '-R' Then
        Begin
          MfmRunFb := True;
        End;
      End;
      If Columns = 3 Then ColumnPos := 25;
      If Columns = 4 Then ColumnPos := 20;
      If Columns = 5 Then ColumnPos := 16;
      If Columns = 6 Then ColumnPos := 14;
      If Columns = 7 Then ColumnPos := 12;
      If Columns = 8 Then ColumnPos := 10;
      If (Not AreaPathOk) Then
      Begin
        Fsplit(Fexpand(ParamStr(0)),D,N,E);
        AreaPath := D+'AREA.DAT';
      End;
      GetAreaTable;
      If NumberOfAreaEntries < Columns Then Columns := NumberOfAreaEntries;
      If (Not OutputSelected) Then
      Begin
        ReDirectTo := Console;
        OutputSelected := True;
        AssignCrt(Input); Reset(Input);
        AssignCrt(Output); Rewrite(Output);
      End;
      SetupScreen;
      If (Not FileAreaPathOk) Then
      Begin
        FileAreaPath := ChooseArea;
        If (FileAreaPath = 'QUIT') Or (FileAreaPath = 'QUITQUICK') Then
        Begin
          Halt(250);
        End;
      End;
    End;
  End;
{========================================================================}
Procedure BuildSkipList;
  Var
    Bslb : Byte;
    InFile : Text;
  Begin
    For Bslb := 1 To MaxSkip Do SkipList[Bslb] := 'ACBDEFGHIJKL';
    Fsplit(Fexpand(ParamStr(0)),D,N,E);
    FindFirst(D+'MFM-SKIP.LST',AnyFile,DirInfo);
    If DosError = 0 Then
    Begin
      Assign(InFile,D+'MFM-SKIP.LST');
      Reset(InFile);
      Bslb := 1;
      While (Not Eof(InFile)) And (Bslb < MaxSkip) Do
      Begin
        ReadLn(InFile,SkipList[Bslb]);
        Inc(Bslb);
      End;
      Close(InFIle);
    End;
  End;
{========================================================================}
Procedure ParseConfigFile;
  Var
    CfgFile : Text;
    CfgStr : String;
  Begin
    DefaultViewer := 'L.COM';
    CompressedFileViewer := 'SHEZ.EXE';
    CompressedFileExt := 'ARCARJLZHPAKSDNZIPZOO';
    PictureFileViewer := 'VPIC.EXE';
    PictureFileExt := 'GIF';
    Fsplit(Fexpand(ParamStr(0)),D,N,E);
    FindFirst(D+'MFM.CFG',AnyFile,DirInfo);
    If DosError = 0 Then
    Begin
      Assign(CfgFile,D+'MFM.CFG');
      Reset(CfgFile);
      While Not Eof(CfgFile) Do
      Begin
        ReadLn(CfgFile,CfgStr);
        CfgStr := UpperString(CfgStr);
        If Pos('DEFAULT VIEWER',CfgStr) > 0 Then DefaultViewer := Copy(CfgStr,18,255);
        If Pos('COMPRESSED FILE VIEWER',CfgStr) > 0 Then CompressedFileViewer := Copy(CfgStr,26,255);
        If Pos('PICTURE FILE VIEWER',CfgStr) > 0 Then PictureFileViewer := Copy(CfgStr,23,255);
        If Pos('COMPRESSED FILE EXT',CfgStr) > 0 Then CompressedFileExt := Copy(CfgStr,23,255);
        If Pos('PICTURE FILE EXT',CfgStr) > 0 Then PictureFileExt := Copy(CfgStr,20,255);
      End;
      Close(CfgFile);
    End;
  End;
{========================================================================}
Function OkToAdd(InString : String) : Boolean;
  Var
    Otab : Byte;
  Begin
    If (MaxAvail > SizeOf(ListRecord)) Then
    Begin
      OkToAdd := True;
      For Otab := 1 To 10 Do If Pos(SkipList[Otab],UpperString(InString)) = 1 Then OkToAdd := False;
    End
    Else
    Begin
      OkToAdd := False;
    End;
  End;
{========================================================================}
Function CommentEntry : Boolean;
  Begin
    CommentEntry := False;
    If Length(WorkString) = 0 Then CommentEntry := True;
    If Copy(WorkString,1,1) = ' ' Then CommentEntry := True;
    If Copy(WorkString,1,1) = '-' Then CommentEntry := True;
    If Pos(WorkString[1],Base153) = 0 Then CommentEntry := True;
  End;
{========================================================================}
Procedure FindOrphans;
  Var
    FileFound : Boolean;
    SearchEntry : ListPtr;
  Begin
    FileFound := False; SearchEntry := FirstEntry;
    If FilesBbs Then
    Begin
      While (Not FileFound) And (SearchEntry^.NextEntry <> NIL) Do
      Begin
        If DirInfo.Name = SearchEntry^.FileName Then FileFound := True;
        SearchEntry := SearchEntry^.NextEntry;
      End;
    End;
    If FilesBbs Then
    Begin
      If (Not FileFound) And (DirInfo.Name <> SearchEntry^.FileName) Then
      Begin
        If OkToAdd(DirInfo.Name) Then
        Begin
          New(NewEntry);
          If NumberOfEntries = 0 Then
          Begin
            FirstEntry := NewEntry;
            NewEntry^.PrevEntry := NIL;
            OldEntry := FirstEntry;
          End
          Else
          Begin
            NewEntry^.PrevEntry := OldEntry;
            OldEntry^.NextEntry := NewEntry;
            OldEntry := NewEntry;
          End;
          NewEntry^.TypeOfRecord := Orphan;
          NewEntry^.FileName := DirInfo.Name;
          NewEntry^.FileSize := DirInfo.Size;
          If DirInfo.Name <> 'FILES.BBS' Then
          Begin
            SizeOfFiles := SizeOfFiles + DirInfo.Size;
            Inc(NumberOfFiles);
          End;
          NewEntry^.FileDate := DirInfo.Time;
          NewEntry^.Description := '';
          NewEntry^.Tagged := False;
          Inc(NumberOfEntries);
        End;
      End;
    End
    Else
    Begin
      If Not FileFound Then
      Begin
        If MaxAvail > SizeOf(ListRecord) Then
        Begin
          New(NewEntry);
          NewEntry^.Tagged := False;
          If NumberOfEntries = 0 Then
          Begin
            FirstEntry := NewEntry;
            NewEntry^.PrevEntry := NIL;
            OldEntry := FirstEntry;
          End
          Else
          Begin
            NewEntry^.PrevEntry := OldEntry;
            OldEntry^.NextEntry := NewEntry;
            OldEntry := NewEntry;
          End;
          NewEntry^.TypeOfRecord := Orphan;
          NewEntry^.FileName := DirInfo.Name;
          NewEntry^.FileSize := DirInfo.Size;
          If DirInfo.Name <> 'FILES.BBS' Then
          Begin
            SizeOfFiles := SizeOfFiles + DirInfo.Size;
            Inc(NumberOfFiles);
          End;
          NewEntry^.FileDate := DirInfo.Time;
          NewEntry^.Description := '';
          Inc(NumberOfEntries);
        End;
      End;
    End;
  End;
{========================================================================}
Procedure BuildList;
  Begin
    NumberOfEntries := 0; FilesBbs := True; Altered := False;
    SizeOfFiles := 0; NumberOfFiles := 0;
    Assign(FileList,FileAreaPath+'FILES.BBS');
    FileMode := 64;
    {$I-} Reset(FileList); {$I+}
    If IOresult = 0 Then
    Begin
      AnsiGotoXY(25,1); NewTextColor(White); NewTextBackground(Black);
      AnsiClearToEOL; Write('Loading FILES.BBS ...');
      While Not Eof(FileList) Do
      Begin
        ReadLn(FileList,WorkString);
        If OkToAdd(WorkString) Then
        Begin
          Inc(NumberOfEntries);
          If CommentEntry Then
          Begin
            New(NewEntry);
            NewEntry^.TypeOfRecord := Comment;
            NewEntry^.FileName := '';
            NewEntry^.FileSize := 0;
            NewEntry^.FileDate := 0;
            NewEntry^.Description := WorkString;
            NewEntry^.Tagged := False;
            If NumberOfEntries = 1 Then
            Begin
              FirstEntry := NewEntry;
              NewEntry^.PrevEntry := NIL;
              OldEntry := FirstEntry;
            End
            Else
            Begin
              NewEntry^.PrevEntry := OldEntry;
              OldEntry^.NextEntry := NewEntry;
              OldEntry := NewEntry;
            End;
          End
          Else
          Begin
            New(NewEntry);
            NewEntry^.Tagged := False;
            If NumberOfEntries = 1 Then
            Begin
              FirstEntry := NewEntry;
              NewEntry^.PrevEntry := NIL;
              OldEntry := FirstEntry;
            End
            Else
            Begin
              NewEntry^.PrevEntry := OldEntry;
              OldEntry^.NextEntry := NewEntry;
              OldEntry := NewEntry;
            End;
            If Pos(' ',WorkString) = 0 Then
            Begin
              NewEntry^.FileName := UpperString(WorkString);
            End
            Else
            Begin
              NewEntry^.FileName := UpperString(Copy(Copy(WorkString,1,Pos(' ',WorkString)-1),1,12));
            End;
            FindFirst(FileAreaPath+NewEntry^.FileName,AnyFile,DirInfo);
            If DosError = 0 Then
            Begin
              NewEntry^.TypeOfRecord := FileRecord;
              NewEntry^.FileSize := DirInfo.Size;
              SizeOfFiles := SizeOfFiles + DirInfo.Size;
              Inc(NumberOfFiles);
              NewEntry^.FileDate := DirInfo.Time;
              If Pos(' ',WorkString) = 0 Then
              Begin
                NewEntry^.Description := '';
              End
              Else
              Begin
                NewEntry^.Description := LtrimRtrim(Copy(WorkString,Pos(' ',WorkString)+1,MaxDescLength));
              End;
            End
            Else
            Begin
              NewEntry^.TypeOfRecord := Offline;
              NewEntry^.FileSize := 0;
              NewEntry^.FileDate := 0;
              If Pos(' ',WorkString) = 0 Then
              Begin
                NewEntry^.Description := '';
              End
              Else
              Begin
                NewEntry^.Description := LtrimRtrim(Copy(WorkString,Pos(' ',WorkString)+1,MaxDescLength));
              End;
            End;
          End;
        End;
      End;
      Close(FileList);
      NewEntry^.NextEntry := NIL;
      If NumberOfEntries = 0 Then FilesBbs := False;
    End
    Else
    Begin
      FilesBbs := False;
    End;
    FindFirst(FileAreaPath+'*.*',Archive,DirInfo);
    If DosError = 0 Then FindOrphans;
    While DosError = 0 Do
    Begin
      NewEntry^.NextEntry := NIL;
      FindNext(DirInfo);
      If DosError = 0 Then FindOrphans;
    End;
    LastEntry := NewEntry;
    LastEntry^.NextEntry := NIL;
    StackEntry := NIL; KillEntry := NIL;
    AnsiGotoXY(25,1); AnsiClearToEOL;
  End;
{========================================================================}
Function Bytes(NumberOfBytes : LongInt) : S8;
  Var
    TempString : S8;
  Begin
    If NumberOfBytes < 1024 Then
    Begin
      TempString := MyStr(NumberOfBytes,4)+'K';
    End
    Else
    Begin
      Str(NumberOfBytes/1024:3:1,TempString);
      TempString := TempString+'M';
    End;
    Bytes := TempString;
  End;
{========================================================================}
Procedure SetupScreen;
  Begin
    NewTextColor(White); NewTextBackground(Black);
    AnsiClearScreen; AnsiGotoXY(24,1);
    NewTextColor(Black); NewTextBackground(Cyan);
    Write(Pgmid+'     ^Q=quit ?=help');
    NewTextColor(White); NewTextBackground(Black);
  End;
{========================================================================}
Procedure ReDrawScreen;
  Begin
    SetupScreen; DisplayScreen;
  End;
{========================================================================}
Begin
End.
{========================================================================}
